/*-------------------------------------
  * fichier:        geiger1.c
  * objet:          programme principal
  * Auteur:         Pascal Chour
  * Cr:           version 1.0 - 2014-03-01
  * Copyright:      (C) Pascal Chour (message_AT_pascalchour_DOT_fr) 
  * version:        1.10
  * date:           01/12/2015
  * version:        1.20
  * date:           10/12/2015
  * version:        1.21
  * date            25/12/2015
  * version:        1.22
  * date            31/12/2015
  * version:        1.3
  * date            22/04/2016 Pour PCGM5

----------------------------------------
Version 1.0
Programmes de tests pour se familiariser avec le processeur.

Version 1.1
Programme de compteur Geiger pour Olimex MSP430-449 STK2.
L'objectif est de minimiser la consommation (en moyenne < 100A).
Le comptage des impulsions se fait via un circuit d'horloge NXP qui consomme trs peu.
Dans les grandes lignes, le programme fonctionne de la faon suivante : 

- le processeur est normalement en sommeil (LPM3)
- il est rveill toutes les secondes.
- il vrifie si on fonctionne avec batterie (E_ALIM_EXTERNE = false ou
  en USB (E_ALIM_EXTERNE = true)
- il vrifie si une alarme (E_ALARME = vrai) est en cours (comptage par mn suprieur  un seuil

Si E_ALIM_EXTERNE = vrai ou si si E_ALARME = vrai ou toutes les 10 secondes,
il effectue le relev du compteur et les calculs correspondant puis affiche les rsultats.

Une fois les traitements termin, il sort de l'interruption.

Le programme principal est une boucle sans fin. Elle est rveille par le programme
d'interruption toutes les secondes.

Elle vrifie si l'utilisateur appuie sur un bouton. Si oui (BTN_MENU), le programme passe
en mode dialogue (E_DIALOGUE=vrai). Il sort de ce mode, soit par action de l'utilisateur, soit
par dtection d'inactivit.

Si E_DIALOGUE= faux et si E_ALARME = faux et si E_ALIM_EXTERNE = faux, alors, il repasse
en sommeil (LPM3).

Les E/S (UART) ne sont disponibles qu'en mode E_ALIM_EXTERNE. 

Version 1.2
- Ajout d'un tat Erreur pour signaler des problmes internes (I2C...). 
- Modification des codes retour du PCF8583
- Quelques simplifications

Version 1.21
- Conversion factor pour SBM20 par dfaut
- envoie du modle de compteur 
- gestion n srie du compteur
- mmorisation dernire valeur tension batterie et gestion commande pour interrogation par GeigerPC

Version 1.22
- modification/amlioration du traitement des boutons poussoirs.

Version 1.3
- version du programme pour PCGM5
- utilisation d'un compteur du MSP430 pour le comptage des impulions. Suppression
du PCF8583.

*/

#include <msp430x44x.h>
#include <in430.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <signal.h>
#include <math.h>
#include <ctype.h>
#include "lcd.h"
#include "geigerutil.h"
#include "geigerutilasm.h"
#include "common.h"

/* Variables d'tat du systme (globales) */

  enum T_ETAT_MENU {MENU_REPOS, MENU_LED, MENU_BUZ, MENU_LIMIT, MENU_UNIT,MENU_ALARME,MENU_24H};
  enum T_ETAT_MENU ETAT_MENU;
  const char s_led_on[] = "LED  ON";
  const char s_led_off[] = "LED OFF";
  const char s_buz_on[]  = "BUZ  ON";
  const char s_buz_off[] = "BUZ OFF";
  const char s_sievert[] = "SIEVERT";
  const char s_cpt_mn[]  = "CPT  PM";
  const char s_limite[]  = "LIM%4.1d";
  const char s_efface[]  = "       ";
  
  unsigned int countMoyenneMin[periode_par_minute]; // nb de periodes par minutes
  int countMoyenneHeure[25];         // nb d'heures +1 par jours
  unsigned int nbMinutes24h;         // nb de minutes coules depuis le dbut du comptage limit   24h00
  unsigned int nbMinutes;            // nb de minutes pour le passage des heures
  unsigned int nbtipmn;              // index courant dans tableau comptage par minutes
  unsigned int nbtiph;               // index courant dans tableau comptage par jours
  unsigned int alarme_cpt;           // compteur d'alarme
  unsigned int compte_inactivite;    // compteur d'inactivit  pour sortir du mode E_DIALOGUE
  unsigned int jauge_batterie;       // valeur de la jauge de la batterie
  unsigned long countParJour;
  unsigned int countParMinute;
  unsigned int seuil_alarme;         // valeur du seuil  partir duquel on gnre une alarme
  unsigned int E_ETAT;               // Etat courant du compteur 
  unsigned int last_count;           // dernire valeur du compteur PCF8583 lue 
  unsigned int last_alarm;           // valeur de la dernire alarme 
  float conversion_factor = 0.0057;  // facteur de conversion courant (SBM20)
  unsigned int derniere_erreur;      // dernire erreur interne du compteur (code, cf defines)
  unsigned int voltage;              // dernire tension batterie mesure

  bool_t    alarme_led;              // vrai si on fait cligonter la LED si alarme */
  bool_t    alarme_buz;              // vrai si on active le buzzer si alarme */
  bool_t    affiche_sievert;         // vrai si on affiche en sievert, sinon, comptage
  bool_t    bascule_1s;              // change d'tat toutes les secondes.
  bool_t    last_bascule_1s;         // dernier tat de bascule_1s.
  bool_t    bascule_arrows;          // pour faire cligonter les flches ou le signe -. Indicateur d'activit
  bool_t    memoire_alarme;          // mmorise le fait qu'il y a eu une alarme
  
  struct trecept recept;

/**
 * Switches red LED on/off.
 *
 * @author Peter Ivanov
 *
 * Input:
 *    @param on TRUE: switch on, FALSE: switch off.
 */
void setStatusLed (bool_t on)
{
    if (on)
    {
        P1OUT &= ~STATUS_LED; // switch on status LED
    }
    else  
    {
        P1OUT |= STATUS_LED;  // switch off status LED
    }
}

/**
 * Switches buzzer on or off. You should frequently switch on and off to 
 * generate a sound.
 *
 * @author Peter Ivanov
 *
 * Input:
 *    @param on TRUE: switch on, FALSE: switch off.
* note : le buzzer est un AZ 1340E P LF, 8mA max, Fr = 4000Hz
 */
void setBuzzer (bool_t on)
{
    if (on)
    {
        P1OUT &= ~BIT0;    //P1.2
        P1OUT |= BIT2;     //P1.3
    }
    else
    {
        P1OUT |= BIT0;     //P1.2
        P1OUT &= ~BIT2;    //P1.3
    }
}


/**
 * Gnre un ou bip de frquence fixe
 *
 * Auteur : Pascal Chour
 *
 * En entre : dure du son en ms.
 */
void sound (unsigned int duree)
{
    while (--duree)
    {
        setBuzzer (TRUE);
        delayMs(1);
        setBuzzer (FALSE);
        setBuzzer (TRUE);
        setBuzzer (FALSE);
    }
}


/*
 mise  jour afficheur partie comptage et alimentation
 lorsqu'on est en mode USB.
 Auteur : Pascal Chour

*/

void maj_afficheur_compteur()
{
  
  float radiationValueMinute; // pour affichage en sievert
  
  if (!(E_ETAT & E_DIALOGUE)) // si DIALOGUE, on n'affiche pas certaines infos de comptage
  {
    if (derniere_erreur == ERR_pas_erreur) { // si pas de code d'erreur
    // affichage par minute sur afficheur principal
    // La valeur en Sievert est gale  la moyenne du comptage par 
    // minute multiplie par un facteur de conversion.
      if (affiche_sievert) {
        radiationValueMinute = countParMinute*conversion_factor;
        LCD_printfsp("SV%6.4f", radiationValueMinute);
      } else { // affichage en comptage
        LCD_printf("PM%5.1d",countParMinute);
      }
    } else { // si code d'erreur
      LCD_printf("ERR%4.1d",derniere_erreur); // on affiche le code  la place de la mesure
    }
  }
  
  
    /* affichage moyenne par jour sur afficheur clock */

  LCD_writeClockInt (countParJour, FALSE);
  
  /* affichage tat batterie mise  jour par ailleurs */
  LCD_showBatteryLevel(jauge_batterie);


#ifdef __FIX_PCB_BARGRAPH
  /* affichage bargraph chelle log */
  if (countParMinute == 0) {
     LCD_bargraph (0,TRUE);
  } else {
    if (countParMinute < 4) {
     LCD_bargraph (1,TRUE);
    } else {
      if (countParMinute < 8) {
       LCD_bargraph (2,TRUE); 
      } else {
        if (countParMinute < 16) {
         LCD_bargraph (3,TRUE); 
        } else {
          if (countParMinute < 32) {
           LCD_bargraph (4,TRUE); 
          } else {
            if (countParMinute < 64) {
             LCD_bargraph (5,TRUE); 
            } else {
              if (countParMinute < 128) {
               LCD_bargraph (6,TRUE); 
              } else {
                if (countParMinute < 256) {
                 LCD_bargraph (7,TRUE); 
                } else {
                  if (countParMinute < 512) {
                   LCD_bargraph (8,TRUE); 
                  } else {
                    if (countParMinute < 1024) {
                     LCD_bargraph (9,TRUE); 
                    } else {
                      LCD_bargraph (10,TRUE); 
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  bascule_arrows = !bascule_arrows;
  if (bascule_arrows) {
    LCD_minus(TRUE);
  } else {
    LCD_minus(FALSE);
  }
#else
  /* on utilise les flches pour signaler activit */
  bascule_arrows = !bascule_arrows;
  if (bascule_arrows) {
    LCD_showArrows(LCD_ARROW_ALL);
  } else {
    LCD_clearArrows(LCD_ARROW_ALL);
  }
#endif // __FIX_PCB_BARGRAPH     
  
  
}
      
      
 

/*
procdure d'interruption (watchdog = WDT_VECTOR) appele chaque seconde.

Si tat diffrent de E_DIALOGUE, E_ALARME et E_ALIM_EXTERNE,
alors, la routine sort quasiment immdiatement sauf toutes les
10 secondes o elle fait les traitements suivants :
- Met  jour le comptage (appel au PCF8583 et mise  jour des variables
- Met  jour l'affichage associ au compteur et  la batterie
la routine vrifi systmatiquement l'tat : 
E_DIALOGUE : sort du mode si inactivit
E_ALARME : sort du mode si alarme depuis plus de 20 s

Auteur : Pascal Chour
*/
#pragma vector=WDT_VECTOR
__interrupt void watchdog_isr( void )
{
    unsigned int current_count;
    bool_t depassement;  // vrai si dpassement de capacit de comptage.     
   
    
    LPM3_EXIT;
    
    /* traitements  faire chaque seconde */
    bascule_1s = !bascule_1s;
    compte_inactivite++;
    alarme_cpt++;

#ifdef __FIX_PCB_BARGRAPH
    if (memoire_alarme) {
      if (bascule_1s) {
         LCD_blink(TRUE); // on autorise l'affichage
      } else {
         LCD_blink(FALSE); // on teint tous les segments
      }
    }
#else
    if (memoire_alarme) {
      if (bascule_1s) {
         LCDM1 = 0x0F; // on allume tous les segments de la jauge 
      } else {
         LCDM1 = 0x00; // on teint tous les segments
      }
    }
#endif //__FIX_PCB_BARGRAPH
    
    /* on s'autorise un test rapide des boutons car 10s, c'est long... */
    if (~P3IN & BTN_MASK) { // si on a appuy sur quelque chose
      compte_inactivite = 0; // il y a de l'activit
      E_ETAT &= ~E_ALARME;  // plus d'alarme
      derniere_erreur = ERR_pas_erreur;  // on rinitialise le code d'erreur, tant pis si on tat en mode dialogue
      memoire_alarme = FALSE; // acquittement alarme, on supprime l'indicateur    
#ifdef __FIX_PCB_BARGRAPH
      LCD_blink(TRUE); // plus de clignotement => bit Blink = TRUE !
#else      
      LCDM1 = 0x00; // on teint tous les segments de la jauge 
#endif        
      E_ETAT |= E_REVEIL; // on force un rveil
    }    

    /* on vrifie si on doit lire les compteurs et afficher */
    countMoyenneMin[nbtipmn] = 0;
    if (((nbtipmn % TIMER_NORMAL) == 0) || (E_ETAT & (E_ALARME | E_ALIM_EXTERNE))) {
      E_ETAT |= E_REVEIL; // pour que le dormeur se rveille

      current_count = TAR; /* lecture du compteur courant */
      if (current_count < last_count) {
        countMoyenneMin[nbtipmn] = (0xFFFF-last_count)+current_count;
      } else {
        countMoyenneMin[nbtipmn] = current_count-last_count;
      }
      last_count = current_count;
      // Nb de comptage par minutes. Il faut attende une minute pour que
      // le comptage soit valide. 
      countParMinute = 0;
      for (int i = 0; i < periode_par_minute; i++) {  
        countParMinute = countParMinute+countMoyenneMin[i];
      } 
    }
    

    // mise   jour de l'index du compteur par priode de temps    
    nbtipmn = nbtipmn+1;
    
    if (((nbtipmn % TIMER_NORMAL) == 0) || (E_ETAT & (E_ALARME | E_ALIM_EXTERNE)) ||
       (nbtipmn >= periode_par_minute)) {

      if (nbtipmn >= periode_par_minute) {
        // sinon, on vient de passer une minute donc on revient au dbut du tableau (tableau circulaire)
        nbtipmn = 0;
        
        if (E_ETAT & E_ALIM_EXTERNE) {E_ETAT |= E_EMET_COMPTAGE;}
        
        // On calcule combien de minutes se sont coules sur la priode glissante courante de 24h. 
        // Au dbut du comptage, le nb de minutes volue de 0   24*60. Ce nb est   mmoriser
        // pour permettre le calcul de la moyenne alors que l'on n'a pas atteind 24H. Lorsque le
        // compteur  atteint 24H (24*60), une journe complte a t compte. On peut donc calculer la moyenne
        // sur les dernires 24H. 
        // Note : Dans le tableau de comptage par heure, la valeur 0xFFFF correspond  un dbordement.
        // Tant qu'une valeur 0xFFFF est prsente dans le tableau, la moyenne sur 24h00 n'est plus calculable.
        // On affiche alors *****.
        if (nbMinutes24h <= nb_minutes_par_24h) {
         nbMinutes24h = nbMinutes24h+1;
        } 
        nbMinutes = nbMinutes+1;

        // On calcule ici le nombre d'impulsion sur 24H glissant
        if (countMoyenneHeure[nbtiph] != 0xFFFF) {   // si pas dbordement
          countMoyenneHeure[nbtiph] += countParMinute;
          if (countMoyenneHeure[nbtiph] < 0) {
            countMoyenneHeure[nbtiph] = 0xFFFF;      // dbordement
          }
        }

        countParJour = 0;   
        depassement = FALSE;            // pas d'overflow
        for (int i = 0; i < 25;i++) {
          if (countMoyenneHeure[i] == (0xFFFF)) {
            depassement = TRUE;
          }
          countParJour = countMoyenneHeure[i]+countParJour;
        }
        if (!depassement) {
          if (nbMinutes24h <= nb_minutes_par_24h) { // si on n'a pas encore fait 24H
            countParJour = countParJour / nbMinutes24h;
          }
          else { // on a fait plus de 24h
            countParJour = countParJour - countMoyenneHeure[nbtiph]; // on vire l'heure en cours
            countParJour = countParJour / nb_minutes_par_24h;
          }
        }

        if (nbMinutes > 60) {
          nbMinutes = 0;
          nbtiph = nbtiph+1;
          if (nbtiph > 24) {
            nbtiph = 0;
            if (E_ETAT & E_ALIM_EXTERNE) {E_ETAT |= E_TEST_BAT;} // pour que le dormeur se rveille
          }
          countMoyenneHeure[nbtiph] = 0;
        }
      }

      if (E_ETAT & E_ALIM_EXTERNE) {
         jauge_batterie = jauge_batterie+1;
         if (jauge_batterie > 3) {jauge_batterie = 0;}
      }
      maj_afficheur_compteur();
    }
}


/*
 Gre le menu du compteur. En entre, le bouton sur lequel
 on a appuy. Procdure non bloquante.
 Auteur : Pascal Chour

*/

void MENU_automate(unsigned int buttonPressed)
{
  switch (ETAT_MENU)
        {
            case MENU_REPOS:
              if (buttonPressed == BTN_MENU) {
                E_ETAT |= E_DIALOGUE;
                if (alarme_led) {
                  LCD_write(s_led_on,7);
                } else {
                  LCD_write(s_led_off,7);
                }
                ETAT_MENU = MENU_LED;
              } else {
                if (buttonPressed == BTN_UP) {
                  E_ETAT |= E_DIALOGUE;
                  if (affiche_sievert) {
                     LCD_printfsp("JR%6.4f", ((float)countParJour*conversion_factor));
                  } else { // affichage en comptage
                     LCD_printf("JR%5.1d",countParJour);
                  }
                  ETAT_MENU = MENU_24H;
                } else {
                  if (buttonPressed == BTN_DOWN) {
                    E_ETAT |= E_DIALOGUE;
                    if (affiche_sievert) {
                       LCD_printfsp("AL%6.4f", ((float)last_alarm*conversion_factor));
                    } else { // affichage en comptage
                       LCD_printf("AL%5.1d",last_alarm);
                    }
                    ETAT_MENU = MENU_ALARME;
                  }
                }
              }
              break;
              
            case MENU_LED:
              if (buttonPressed == BTN_MENU) {
                E_ETAT &= ~E_DIALOGUE;
                LCD_write(s_efface,7);
                maj_afficheur_compteur();               
                ETAT_MENU = MENU_REPOS;
              } else {
                if ((buttonPressed == BTN_UP) || (buttonPressed == BTN_DOWN)) {
                  alarme_led = !alarme_led;
                  if (alarme_led) {
                    LCD_write(s_led_on,7);
                  } else {
                    LCD_write(s_led_off,7);
                  }
                } else {
                  ETAT_MENU = MENU_BUZ;
                  if (alarme_buz) {
                    LCD_write(s_buz_on,7);
                  } else {
                    LCD_write(s_buz_off,7);
                  }
                }
              }
              break;

            case MENU_BUZ:
              if (buttonPressed == BTN_MENU) {
                E_ETAT &= ~E_DIALOGUE;
                LCD_write(s_efface,7);
                maj_afficheur_compteur();               
                ETAT_MENU = MENU_REPOS;
              } else {
                if ((buttonPressed == BTN_UP) || (buttonPressed == BTN_DOWN)) {
                  alarme_buz = !alarme_buz;
                  if (alarme_buz) {
                    LCD_write(s_buz_on,7);
                  } else {
                    LCD_write(s_buz_off,7);
                  }
                } else {
                  ETAT_MENU = MENU_LIMIT;
                  LCD_printf(s_limite,seuil_alarme);
                }
              }
              break;


            case MENU_LIMIT:
              if (buttonPressed == BTN_MENU) {
                E_ETAT &= ~E_DIALOGUE;
                LCD_write(s_efface,7);
                maj_afficheur_compteur();                
                ETAT_MENU = MENU_REPOS;
              } else {
                if (buttonPressed == BTN_UP) {
                  seuil_alarme++;
                  if (seuil_alarme > 999) {seuil_alarme = 999;}
                  LCD_printf(s_limite,seuil_alarme);
                  } else {
                    if (buttonPressed == BTN_DOWN) {
                      seuil_alarme--;
                      if (seuil_alarme < 1) {seuil_alarme = 1;}
                      LCD_printf(s_limite,seuil_alarme);
                    } else {
                      ETAT_MENU = MENU_UNIT;
                      if (affiche_sievert) {
                        LCD_write(s_sievert,7);
                      } else {
                        LCD_write(s_cpt_mn,7);
                      }
                    }
                  }
              }
              break;  

            case MENU_UNIT:
              if ((buttonPressed == BTN_UP) || (buttonPressed == BTN_DOWN)) {
                affiche_sievert = !affiche_sievert;
                if (affiche_sievert) {
                  LCD_write(s_sievert,7);
                } else {
                  LCD_write(s_cpt_mn,7);
                }
              } else {
                E_ETAT &= ~E_DIALOGUE;
                LCD_write(s_efface,7);
                maj_afficheur_compteur();            
                ETAT_MENU = MENU_REPOS;
              }
              break;
              
            case MENU_24H:  
            case MENU_ALARME:
                E_ETAT &= ~E_DIALOGUE;
                LCD_write(s_efface,7);
                maj_afficheur_compteur();            
                ETAT_MENU = MENU_REPOS;
              break;    
 
        }
}



/**
 * Initialisation du systme
 *
 * @author Peter Ivanov 
 * modifications Pascal Chour
 */
void init ()
{
    char s[BUF_MAX];
    WDTCTL = WDTPW | WDTHOLD;                       // stop watchdog timer

    FLL_CTL0 &= ~XTS_FLL;                           // XT1 as low-frequency
    // !!!!!!!!!!!
    FLL_CTL0 = XCAP18PF;                            // set load capacitance for 32k xtal
    // !!!!!!!!!!!

    _BIC_SR (OSCOFF);                               // turn on XT1 oscillator

    do                                              // wait in loop until crystal is stable
    {
        IFG1 &= ~OFIFG;
    }
    while (IFG1 & OFIFG);

    FLL_CTL1 &= ~FLL_DIV0;                          // ACLK = XT1
    FLL_CTL1 &= ~FLL_DIV1;

    IFG1 &= ~OFIFG;                                 // clear osc. fault int. flag

    FLL_CTL1 &= ~SELM0;                             // set DCO as MCLK
    FLL_CTL1 &= ~SELM1;

    LCD_init ();

    // Initialize IO port
    P1SEL = BIT5;            //p1.5 is 32768Hz
    P1DIR = BIT5 | BIT3 | BIT0 | BIT2;                        //BUZ,LED are outputs
    
    LCD_clear ();              // turn off all LCD segments
    LCD_write("P.CHOUR",7);
    sound (100);               // test bip
    setStatusLed (TRUE);       // test diode
    delayMs (300);
    setStatusLed (FALSE);
    delayMs (300);
    setStatusLed (TRUE);
    delayMs (300);
    setStatusLed (FALSE);
    sprintf(s,"%s%s","VER ",version_pgm);
    LCD_writesp(s,8);
    delayMs(1000);
    LCD_write("INIT   ",7);

    /* tats du systme */
    E_ETAT = 0;                    // pas de mode particulier en cours
    ETAT_MENU = MENU_REPOS;        // tat du menu des options du compteur
        
    alarme_buz = FALSE;            // alarme LED dsactive
    alarme_led = FALSE;            // alarme buzzer dsactive
    seuil_alarme = alarme_defaut;  // dfinit le seuil de dclenchement de l'alarme
    
    affiche_sievert = FALSE;       // affiche le comptage brut
    alarme_cpt = 0;                // compteur dure alarme
    compte_inactivite = 0;         // compteur d'inactivit
    nbMinutes = 0;                 // nb minutes pour passages heures
    nbtipmn = 0;                   // index courant dans tableau comptage par minutes
    nbtiph  = 0;                   // index courant dans tableau comptage par jours   
    nbMinutes24h = 0;              // nb de minutes coules depuis le dbut du comptage limit   24h00
    countParMinute = 0;            // pas de comptage par mn valide
    countParJour = 0;              // pas de comptage par jour valide
    last_count = 0;                // dernire valeur du compteur
    bascule_1s = FALSE;            // il faut lui donner un valeur...
    last_bascule_1s = FALSE;       // mme tat que bascule_1s
    bascule_arrows = FALSE;        // pas de flches affiches
    memoire_alarme = FALSE;        // pas d'alarme mmorise
    last_alarm = 0;                // dernire alarme mmorise
    derniere_erreur = ERR_pas_erreur; // il n'y a pas encore d'erreur...
    
    P2SEL &= ~0x04;                 // P2SEL GPIO pour test alim externe
    P2DIR &= ~0x04;                 // DIR = 0 => input    
    UART0_initbuffer();             // prpare le buffer pour la rception
    
   
    // principe de comptage : un tableau comportant "periode_par_minute" positions
    // est utilis pour calculer une moyenne glissante. Ex : si une priode de comptage est
    // 10s, alors, il y a 6 priodes de comptage par minute. Ce tableau est initialis
    //  0 au dmarrage.
    // Il faut attendre une minute pour que la moyenne de comptage soit
    // correcte.    
    for (int i = 0; i < periode_par_minute;i++) {
      countMoyenneMin[i] = 0;     
    }
    // on initialise les tableaux pour comptage par heure sur 24H
    for (int i = 0; i < 25; i++) {
      countMoyenneHeure[i] = 0; 
    }     

    // P1.5 DIR = 0 => input et dselectionn
    P1DIR &= ~BIT5;                 

    // RAZ timer A
    TACTL = TACLR;
    // On initialise le compteur, (TASSEL_0) entre =  TACLK, (ID_0) diviseur = 1, (MC_2) Continuous mode, interruption dsactive
    TACTL = TASSEL_0+ID_0+MC_2;    


    
    jauge_batterie = niveau_batterie(&voltage);  // niveau de la batterie -> jauge batterie
    maj_afficheur_compteur();    

    WDTCTL = WDT_ADLY_1000;     // WDT interrupt in every second
    IE1 |= WDTIE;
    _EINT ();

    
}

/**
 * Boucle sans fin. Programme principal.
* Auteur : Pascal Chour

 *
 */
void run ()
{
    unsigned int buttonPressed;        // Donne les poussoirs actifs
    buttonPressed = 0;                // pas de poussoirs actifs

    while (1)
    {
      if (E_ETAT) { // y a-t-il un tat qui justifie de faire quelque chose ?
  
         /* on vrifie si on sort du mode dialogue pour cause d'inactivit */
         if (E_ETAT & E_DIALOGUE) {
           if (compte_inactivite > INACTIVITE_MAX) {
             E_ETAT &= ~E_DIALOGUE;
             ETAT_MENU = MENU_REPOS;
           }
         }
  
         /* on vrifie si on sort du mode alarme */
         if (E_ETAT & E_ALARME) {
           if (bascule_1s != last_bascule_1s) {
             if (alarme_led) { // Alarme LED actif ?
                setStatusLed (TRUE);               
                delayMs (15);   // alors, elle a t allume dans la routine d'IT
                setStatusLed (FALSE); // on l'teint.
             }
             if ((alarme_buz) && (bascule_1s)) { // Alarme buzzer actif ?
                 sound(33); // Oui, on gnre un son ici.
                 delayMs(33);
                 sound(33); // Oui, on gnre un son ici.
             }
           }
           last_bascule_1s = bascule_1s;
           if (alarme_cpt > ALARME_MAX) {E_ETAT &= ~E_ALARME;}
         } else { // il n'y a pas d'alarme
           if (countParMinute >= seuil_alarme) { // y -a-til une alarme ?
             E_ETAT |= E_ALARME; // oui, alors, on le signale.
             memoire_alarme = TRUE; // et on le mmorise
             last_alarm = countParMinute; // on conserve cette valeur pour affichage
             alarme_cpt = 0; // et on initialise un compteur pour l'arrt automatique de l'alarme
           }
         }
         
        /* on vrifie si on est aliment en USB */
        if (alim_USB()) {
          if (!(E_ETAT & E_ALIM_EXTERNE)) {  // si on n'tait pas dj en ALIM_EXTERNE
             UART0_enable();                 // alors on initialise l'UART
          }
          E_ETAT |= E_ALIM_EXTERNE;  
          if (E_ETAT & E_EMET_COMPTAGE) {
            EMET_CM(countParMinute);         // on transmet CM chaque minute au PC raccord (s'il y a)
            E_ETAT &= ~E_EMET_COMPTAGE;            
          }
          if (recept.a_traiter) {traite_commande();}
        } else {                             // on n'est pas/plus en alimentation externe 
          if (E_ETAT & E_ALIM_EXTERNE) {     // on tait en alimentation externe est on vient de passer sur batterie
            jauge_batterie = niveau_batterie(&voltage); // on rafraichit jauge_batterie
            LCD_showBatteryLevel(jauge_batterie); // on force l'affichage
            UART0_disable();               // on supprime les E/S
            E_ETAT &= ~E_ALIM_EXTERNE;
            E_ETAT &= ~E_EMET_COMPTAGE;    // au cas o il y aurait une demande d'mission en cours
          } else {   // On n'tait pas en alimentation externe. On en profite pour vrifier s'il faut mettre  jour la tension batterie 
            if (E_ETAT & E_TEST_BAT) { // ce traitement est fait avec une priode longue
              jauge_batterie = niveau_batterie(&voltage); // 450A consomm !
            }
          }
        }
         
        /* Gestion des menus */           
        if (buttonHandler (&buttonPressed)) {// on a appuy sur un bouton ?
           MENU_automate(buttonPressed);
        }
         
        /* on dsactive EMET_COMPTAGE, TEST_BAT et REVEIL_COURT car  ce stade, on est sur d'avoir
           fait les traitements correspondants s'ils taient  faire */

         E_ETAT &= ~E_TEST_BAT;
         E_ETAT &= ~E_REVEIL;          
      }
      
        /* si pas dialogue ou alarme ou alim externe... on se met
         en sommeil jusqu' la prochaine interruption. Sinon, on boucle en continue */
       if (!E_ETAT){
          LPM3; /* on s'endort... */
       }       
    }

}

/**
 * Point d'entre
 */
int main (void)
{
    init ();
    run ();
    return 0;
}

