/*
 * File:        lcd.c
 * Purpose:     LCD handling functions
 * Author:      Peter Ivanov, Muneem Shahriar
 * Modified by:
 * Created:     2007-05-19 11:29:32
 *              2007-10-21 11:34:26 ivanovp {Time-stamp}
 *              2014-03-01 Pascal Chour
 * Last modify: 2015-12-02 Pascal Chour (LCD_write, gestion du point + bug dans affichage niveau batterie)
 *                 Gestion du point dans l'affichage : LCD_writesp qui ne compte pas le point comme
 *                 un caractre. Exemple : soit le texte "VER 6.1"
 *                 LCD_write("VER 6.1")   => VER 6. 1 
 *                 LCD_writesp("VER 6.1") =>  VER 6.1 
 *
 *              2015-12-25 Pascal Chour (Bargraph...)
 *                 Gestion du Bargraph sur PCB modifi. Ajout d'une condition de compilation
 *                 __FIX_PCB_BARGRAPH si PCB modifi. Ajout des fonctions
 *                 LCD_minus et LCD_plus pour un affichage plus correct des "+" et "-".
 *
 * Copyright:   (C) Peter Ivanov, 2007
 * Copyright:   (C) Pascal Chour, 2015
 * Licence:     GPL
*/

/**
 * \file lcd.c
 * \brief LCD handling functions
 * \author Peter Ivanov, Muneem Shahriar
 */
// Contains code from Muneem Shahriar
#include <msp430f449.h>
#include <inttypes.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "lcd.h"


//#define __DEBUG__   1

char *LCD = LCDMEM;

void LCD_init ()
{
    // Initialize LCD
    LCD_clear ();

    TACTL = TASSEL0 | TACLR;                // set timer_A
    LCDCTL = LCDON + LCD4MUX + LCDSG0_7;    // STK LCD 4mux LCD, S0-S39
    //BTCTL = BT_fLCD_DIV64;                // STK LCD freq ACLK/64
    BTCTL = BT_fLCD_DIV128;                 // set LCD frame freq = ACLK
    P5SEL = 0xFC;                           // set Rxx and COM pins for LCD
}

void LCD_clear ()
{
    uint8_t i;
    for (i = 0; i < 20; i++) 
    {
        LCD[i] = 0x00; //clear Sxx
    }
}

void LCD_all ()
{
    uint8_t i;
    unsigned int j;
    for (i = 0; i < 20; i++)
    {
      for (j = 0; j <= 255; j++)
      {
        LCD[i] = j; // turn on all LCD segments
      }
    }
}

void LCD_clearClock ()
{
    uint8_t i;
    for (i = 2; i < 6; i++)
    {
        LCD[i] = 0;
    }
}

void LCD_writeClockInt (uint16_t value, bool_t paddingZeroes)
{
    const uint8_t base = 10;
    uint8_t i;
    if (value <= 9999)
    {
        for (i = 0; i < 4; i++)
        {
            if (paddingZeroes)
            {
                LCD_writeClockDigit (i, value % base);
            }
            else
            {
                if (value)
                {
                    LCD_writeClockDigit (i, value % base);
                }
                else
                {
                    LCD_writeClockDigit (i, 255); // clear all segments
                }
            }
            value /= base;
        }
    }
    else
    {
#ifdef __DEBUG__
        UART_printf ("%s(): value is too big: %i!\r\n", value);
#endif // __DEBUG__
    }
}

void LCD_setClockColon (bool_t on)
{
    if (on)
    {
        LCD[3] |= 0x80;
    }
    else
    {
        LCD[3] &= ~0x80;
    }
}

void LCD_setClockDot (uint8_t position, bool_t on)
{
    if (on)
    {
        switch (position)
        {
            case 0:
                LCD[2] |= 0x80;
                break;
            case 1:
                LCD[4] |= 0x08;
                break;
            case 2:
                LCD[5] |= 0x08;
                break;
            default:
#ifdef __DEBUG__
                UART_printf ("%s(): invalid position: %i!\r\n", __FUNCTION__, position);
#endif // __DEBUG__
                break;
        }
    }
    else
    {
        switch (position)
        {
            case 0:
                LCD[2] &= ~0x80;
                break;
            case 1:
                LCD[4] &= ~0x08;
                break;
            case 2:
                LCD[5] &= ~0x08;
                break;
            default:
#ifdef __DEBUG__
                UART_printf ("%s(): invalid position: %i!\r\n", __FUNCTION__, position);
#endif // __DEBUG__
                break;
        }
    }
}

//#define LCD_CLEAR
#ifdef LCD_CLEAR
#warning LCD_CLEAR option is not recommended!
#endif

void LCD_writeClockDigit (uint8_t position, uint8_t digit)
{
    if (position < 2)
    {
        // LCD digit segments
#define CA (16)
#define CB (1)
#define CC (4)
#define CD (8)
#define CE (64)
#define CF (32)
#define CG (2)
        switch (digit)
        {
            case 0:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CC + CD + CE + CF;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CC + CD + CE + CF;
                LCD[position + 2] &= ~CG;
#endif /* LCD_CLEAR */
                break;
            case 1:
#if LCD_CLEAR
                LCD[position + 2] = CB + CC;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CB + CC;
                LCD[position + 2] &= ~(CA + CD + CE + CF + CG);
#endif /* LCD_CLEAR */
                break;
            case 2:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CG + CE + CD;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CG + CE + CD;
                LCD[position + 2] &= ~(CC + CF);
#endif /* LCD_CLEAR */
                break;
            case 3:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CG + CC + CD;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CG + CC + CD;
                LCD[position + 2] &= ~(CE + CF);
#endif /* LCD_CLEAR */
                break;
            case 4:
#if LCD_CLEAR
                LCD[position + 2] = CF + CB + CG + CC;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CF + CB + CG + CC;
                LCD[position + 2] &= ~(CA + CD + CE);
#endif /* LCD_CLEAR */
                break;
            case 5:
#if LCD_CLEAR
                LCD[position + 2] = CA + CF + CG + CC + CD;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CF + CG + CC + CD;
                LCD[position + 2] &= ~(CB + CE);
#endif /* LCD_CLEAR */
                break;
            case 6:
#if LCD_CLEAR
                LCD[position + 2] = CA + CF + CG + CC + CD + CE;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CF + CG + CC + CD + CE;
                LCD[position + 2] &= ~CB;
#endif /* LCD_CLEAR */
                break;
            case 7:
#if LCD_CLEAR
                LCD[position + 2] = CB + CC + CA;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CB + CC + CA;
                LCD[position + 2] &= ~(CD + CE + CF + CG);
#endif /* LCD_CLEAR */
                break;
            case 8:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CC + CD + CE + CF + CG;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CC + CD + CE + CF + CG;
#endif /* LCD_CLEAR */
                break;
            case 9:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CC + CD + CF + CG;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CC + CD + CF + CG;
                LCD[position + 2] &= ~CE;
#endif /* LCD_CLEAR */
                break;
            case 255: // special code: clear all segments
#if LCD_CLEAR
                LCD[position + 2] = 0;
#else /* LCD_CLEAR */
                LCD[position + 2] &= ~(CA + CB + CC + CD + CE + CF + CG);
#endif /* LCD_CLEAR */
                break;
            default:
#ifdef __DEBUG__
                UART_printf ("%s(): invalid digit: %i!\r\n", __FUNCTION__, digit);
#endif // __DEBUG__
                break;
        }
    }
    else if (position < 4)
    {
#undef CA
#undef CB
#undef CC
#undef CD
#undef CE
#undef CF
#undef CG
        // LCD digit segments
#define CA  (1)
#define CB  (2)
#define CC  (4)
#define CD  (128)
#define CE  (64)
#define CF  (16)
#define CG  (32)
        switch (digit)
        {
            case 0:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CC + CD + CE + CF;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CC + CD + CE + CF;
                LCD[position + 2] &= ~CG;
#endif /* LCD_CLEAR */
                break;
            case 1:
#if LCD_CLEAR
                LCD[position + 2] = CB + CC;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CB + CC;
                LCD[position + 2] &= ~(CA + CD + CE + CF + CG);
#endif /* LCD_CLEAR */
                break;
            case 2:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CG + CE + CD;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CG + CE + CD;
                LCD[position + 2] &= ~(CC + CF);
#endif /* LCD_CLEAR */
                break;
            case 3:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CG + CC + CD;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CG + CC + CD;
                LCD[position + 2] &= ~(CE + CF);
#endif /* LCD_CLEAR */
                break;
            case 4:
#if LCD_CLEAR
                LCD[position + 2] = CF + CB + CG + CC;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CF + CB + CG + CC;
                LCD[position + 2] &= ~(CA + CD + CE);
#endif /* LCD_CLEAR */
                break;
            case 5:
#if LCD_CLEAR
                LCD[position + 2] = CA + CF + CG + CC + CD;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CF + CG + CC + CD;
                LCD[position + 2] &= ~(CB + CE);
#endif /* LCD_CLEAR */
                break;
            case 6:
#if LCD_CLEAR
                LCD[position + 2] = CA + CF + CG + CC + CD + CE;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CF + CG + CC + CD + CE;
                LCD[position + 2] &= ~CB;
#endif /* LCD_CLEAR */
                break;
            case 7:
#if LCD_CLEAR
                LCD[position + 2] = CB + CC + CA;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CB + CC + CA;
                LCD[position + 2] &= ~(CD + CE + CF + CG);
#endif /* LCD_CLEAR */
                break;
            case 8:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CC + CD + CE + CF + CG;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CC + CD + CE + CF + CG;
#endif /* LCD_CLEAR */
                break;
            case 9:
#if LCD_CLEAR
                LCD[position + 2] = CA + CB + CC + CD + CF + CG;
#else /* LCD_CLEAR */
                LCD[position + 2] |= CA + CB + CC + CD + CF + CG;
                LCD[position + 2] &= ~CE;
#endif /* LCD_CLEAR */
                break;
            case 255: // special code: clear all segments
#if LCD_CLEAR
                LCD[position + 2] = 0;
#else /* LCD_CLEAR */
                LCD[position + 2] &= ~(CA + CB + CC + CD + CE + CF + CG);
#endif /* LCD_CLEAR */
                break;
            default:
#ifdef __DEBUG__
                UART_printf ("%s(): invalid digit: %i!\r\n", __FUNCTION__, digit);
#endif // __DEBUG__
                break;
        }
    }
    else
    {
#ifdef __DEBUG__
        UART_printf ("%s(): invalid position: %i!\r\n", __FUNCTION__, position);
#endif // __DEBUG__
    }
}

void LCD_writeInt (int number)  // A cool function that moves number right to left
{
   unsigned int i;              // dummy variable
   unsigned int digit;          // dummy digit       
   char Letter;

   for (i = 0; i < 7; i++)      // Extract each digit in number, put in an integer array, and count total length also
   {
      digit = number % 10;      // digit = the least significant character obtained from number for display
      number = number / 10;     // remove the least significant character from number
      
      switch (digit)            // pass on the right char value to writeLetter function
      {   
          case 0:  Letter = '0'; LCD_writeLetter (i, Letter); break;
          case 1:  Letter = '1'; LCD_writeLetter (i, Letter); break;
          case 2:  Letter = '2'; LCD_writeLetter (i, Letter); break;
          case 3:  Letter = '3'; LCD_writeLetter (i, Letter); break;
          case 4:  Letter = '4'; LCD_writeLetter (i, Letter); break;
          case 5:  Letter = '5'; LCD_writeLetter (i, Letter); break;
          case 6:  Letter = '6'; LCD_writeLetter (i, Letter); break;
          case 7:  Letter = '7'; LCD_writeLetter (i, Letter); break;
          case 8:  Letter = '8'; LCD_writeLetter (i, Letter); break;
          case 9:  Letter = '9'; LCD_writeLetter (i, Letter); break;
      }
      
      if (number == 0)                 // when the number has finally been reduced to zero
      {
          break;                       // break so that LCD doesn't diplay leading zeroes. E.g 234 instead of 0000234
      }
   } 
}

void LCD_printf (const char *fmt, ...)
{
    static va_list valist;
    char buf[LCD_PRINTF_BUF_SIZE];
    
    va_start (valist, fmt);
    vsnprintf (buf, sizeof (buf), fmt, valist);
    va_end (valist);	
    LCD_write (buf, strlen (buf));
}

void LCD_printfsp (const char *fmt, ...)
{
    static va_list valist;
    char buf[LCD_PRINTF_BUF_SIZE+2];
    
    va_start (valist, fmt);
    vsnprintf (buf, sizeof (buf), fmt, valist);
    va_end (valist);	
    LCD_writesp (buf, strlen (buf));
}

void LCD_write (const char *buf, uint8_t length)
{
   uint8_t i;
   int8_t pos;

   for (i = 0, pos = 6; i < length && pos >= 0; i++, pos--) // display str
   {
     if (buf[i] == '.') {LCD_writeLetter (pos, ' ');}
     LCD_writeLetter (pos, buf[i]);  // displays each letter in the str
   }
}

void LCD_writesp (const char *buf, uint8_t length)
{
   uint8_t i;
   int8_t pos;

   pos = 6;
   i = 0;
   while (i < length)
   {
     if ((i < length-1) && (buf[i] == '.')) {
       LCD_writeLetter (pos, buf[i+1]);
       LCD_writeLetter (pos, '.');
       i = i+2;
     } else {
       LCD_writeLetter (pos, buf[i]);
       i = i+1;
     }
     pos--;
   } 
}

#define a       (0x80)    // definitions for LCD seegments on the Olimex LCD. 4-Mux operation is assumed
#define b       (0x40)    // For more details on 4-Mux operation, gather your LCD datasheet, 
#define c       (0x20)    // TI's MSP430F449 User Guide (look for LCD Controller, then 4-Mux),
#define d       (0x01)    // and MSP-449STK-2 schematic. You will need ALL these 3 when defining
#define e       (0x02)    // each number or character. Remember, the Olimex LCD doesn't use a LCD driver!
#define f       (0x08)    // You tell the LCD what characters to display. It's very time consuming!!
#define g       (0x04)
#define h       (0x10)



/*
Affiche les segments de la batterie 
level : 0  3 en fonction du niveau 
*/
void LCD_showBatteryLevel (uint8_t level)
{
  uint8_t LCDM2_new;
  LCDM2_new = LCDM2 & LCD_ARROW_ALL;
  switch (level)
    {
        case 0: LCDM2_new |= d;             break;  // batterie vide
        case 1: LCDM2_new |= d + f;         break;  // batterie faible
        case 2: LCDM2_new |= d + f + g;     break;  // batterie moyenne
        case 3: LCDM2_new |= d + e + f + g; break;  // batterie charge
    }
  LCDM2 = LCDM2_new;
}

#ifdef __FIX_PCB_BARGRAPH

/*
Affiche le bargraph
level : 0  10 en fonction du niveau 
extremits  vrai affiche les [..]
Procdure utilisable uniquement si le PCB
a t modifi : suppression des rsistances
R10 et R9, coupure des pattes 42 et 35 de
l'afficheur LCD, strap entre les points 
(42 et 40) et (35 et 37) du PCB.
*/
void LCD_bargraph (uint8_t level,bool_t extremites)
{
  uint8_t LCDM1_new, LCDM2_new;
  
  if (extremites) {
    LCDM1_new = 0x08;
    LCDM2_new = (LCDM2 & 0x0F) | 0x10;
  } else {
    LCDM1_new = 0;
    LCDM2_new = LCDM2 & 0x0F;
  }
  switch (level)
    {
        case 1: LCDM1_new |= 0x04;  break;  
        case 2: LCDM1_new |= 0x06;  break; 
        case 3: LCDM1_new |= 0x07;  break; 
        case 4: LCDM1_new |= 0x17;  break; 
        case 5: LCDM1_new |= 0x37;  break;  
        case 6: LCDM1_new |= 0x77;  break;  
        case 7: LCDM1_new |= 0xF7;  break; 
        case 8: LCDM1_new |= 0xF7;  LCDM2_new |= 0x80;  break;  
        case 9: LCDM1_new |= 0xF7;  LCDM2_new |= 0xC0;  break; 
        case 10: LCDM1_new |= 0xF7;  LCDM2_new |= 0xE0;  break;  
    }
  LCDM1 = LCDM1_new;
  LCDM2 = LCDM2_new;
}
#else
void LCD_showArrows (uint8_t arrows)
{
    LCDM2 |= arrows;
}

void LCD_clearArrows (uint8_t arrows)
{
    LCDM2 &= ~arrows;
}
#endif

/* affichage ou extinction du "-" */
void LCD_minus(bool_t on)
{
  if (on) {
    LCDM20 |= h;
  } else {
    LCDM20 &= ~h;
  }
}

/* affichage ou extinction du "+" */
void LCD_plus(bool_t on)
{
  if (on) {
    LCDM20 |= a;
  } else {
    LCDM20 &= ~a;
  }
}

/* Gestion du clignotement de l'afficheur */
void LCD_blink(bool_t on)
{
  if (on) {
    LCDCTL |= LCDSON;
  } else {
    LCDCTL &= ~LCDSON;
  }
}

    

void LCD_setColon (uint8_t position, bool_t on)
{
    position++;
    // DO NOT PLAY WITH THE CODE BELOW ------------------------------------------------------------------------------
    if (position == 1)
    { 
        // this is position adjustment for compatibility.
        position = position + 6;
    }
    else if (position == 2 || position == 3 || position == 4 || position == 5 || position == 6 || position == 7)
    {
        // adjust position  
        position = ((position * 2) - 1) + 6; 
    }
    // --------------------------------------------------------------------------------------------------------------
    if (on)
    {
        LCD[position] |= h;
    }
    else
    {
        LCD[position] &= ~h;
    }
}

// writes a single character on the LCD. User can specify position as well
void LCD_writeLetter (uint8_t position, char letter)
{
    position++;
    // DO NOT PLAY WITH THE CODE BELOW ------------------------------------------------------------------------------
    if (position == 1)
    { 
        // this is position adjustment for compatibility.
        position = position + 6;
    }
    else if (position == 2 || position == 3 || position == 4 || position == 5 || position == 6 || position == 7)
    {
        // adjust position  
        position = ((position * 2) - 1) + 6; 
    }
    // --------------------------------------------------------------------------------------------------------------
    if (letter >= 'a' && letter <= 'z')
    {
        letter &= 0xDF; // uppercase
    }
    switch (letter)                                  
    {
        case ' ':  LCD[position-1] = 0;                  LCD[position] = 0;                break;
        // letter  // LCDM7                           // LCDM8                          // End
        case 'A':  LCD[position-1] = a + b + c + e;      LCD[position] = b + c + g;        break;
        case 'B':  LCD[position-1] = c + h + e;          LCD[position] = b + c + g;        break;  
        case 'C':  LCD[position-1] = a + h;              LCD[position] = b + c;            break;  
        case 'D':  LCD[position-1] = b + c + h + e;      LCD[position] = c + g;            break;     
        case 'E':  LCD[position-1] = a + h + e;          LCD[position] = b + c + g;        break;  
        case 'F':  LCD[position-1] = a;                  LCD[position] = b + c + g;        break;  
        case 'G':  LCD[position-1] = a + c + h + e;      LCD[position] = b + c;            break; 
        case 'H':  LCD[position-1] = b + c + e;          LCD[position] = b + c + g;        break;  
        case 'I':  LCD[position-1] = a + h + f;          LCD[position] = d;                break;  
        case 'J':  LCD[position-1] = b + h + c;          LCD[position] = c;                break;  
        case 'K':  LCD[position-1] = d + g;              LCD[position] = b + c + g;        break;  
        case 'L':  LCD[position-1] = h;                  LCD[position] = b + c ;           break;    
        case 'M':  LCD[position-1] = b + c + g;          LCD[position] = b + c + f;        break;    
        case 'N':  LCD[position-1] = b + c + d;          LCD[position] = b + c + f;        break;
        case 'O':  LCD[position-1] = a + b + c + h;      LCD[position] = b + c;            break;
        case 'P':  LCD[position-1] = a + b + e;          LCD[position] = b + c + g;        break;
        case 'Q':  LCD[position-1] = a + b + c + h + d;  LCD[position] = b + c;            break;
        case 'R':  LCD[position-1] = a + b + d + e;      LCD[position] = b + c + g;        break;
        case 'S':  LCD[position-1] = a + c + h + e;      LCD[position] = b + g;            break;
        case 'T':  LCD[position-1] = a + f + b;          LCD[position] = d + b;            break;
        case 'U':  LCD[position-1] = b + c + h;          LCD[position] = b + c;            break;
        case 'V':  LCD[position-1] = g;                  LCD[position] = b + c + e;        break;
        case 'W':  LCD[position-1] = b + c + d;          LCD[position] = b + c + e;        break;
        case 'X':  LCD[position-1] = d + g;              LCD[position] = e + f;            break;
        case 'Y':  LCD[position-1] = b + c + h + e;      LCD[position] = f;                break;
        case 'Z':  LCD[position-1] = a + h + g;          LCD[position] = e;                break;      

             // number  // LCDM7                              // LCDM8                          // END
        case '0':  LCD[position-1] = a + b + c + h + g;  LCD[position] = b + c + e;        break; 
        case '1':  LCD[position-1] = b + c;              LCD[position] = 0;                break; 
        case '2':  LCD[position-1] = a + b + e + h;      LCD[position] = c + g;            break; 
        case '3':  LCD[position-1] = a + b + c + e + h;  LCD[position] = g;                break; 
        case '4':  LCD[position-1] = b + c + e;          LCD[position] = b + g;            break; 
        case '5':  LCD[position-1] = a + c + h + e;      LCD[position] = b + g;            break; 
        case '6':  LCD[position-1] = a + c + h + e;      LCD[position] = b + c + g;        break; 
        case '7':  LCD[position-1] = a + b + c;          LCD[position] = 0;                break;
        case '8':  LCD[position-1] = a + b + c + e + h;  LCD[position] = b + c + g;        break; 
        case '9':  LCD[position-1] = a + b + c + e + h;  LCD[position] = b + g;            break; 

        case '/':  LCD[position-1] = g;                  LCD[position] = e;                break;
        case '\\': LCD[position-1] = d;                  LCD[position] = f;                break;
        case '\'': LCD[position-1] = g;                  LCD[position] = 0;                break;
        case ',':  LCD[position-1] = 0;                  LCD[position] = e;                break;
        case '>':  LCD[position-1] = 0;                  LCD[position] = e + f;            break; 
        case '<':  LCD[position-1] = d + g;              LCD[position] = 0;                break; 

                   // others
        case '.':                                        LCD[position] |= h;               break;  // decimal point
        case ':':                                        LCD[position] |= a;               break;  // colon

        case '+':                                        LCDM20= a;                        break;  // plus sign
        case '-':                                        LCDM20= h;                        break;  // minus sign
    }
}

