/*
 * File:        lcd.h
 * Purpose:     LCD handling functions header
 * Author:      Peter Ivanov
 * Modified by:
 * Created:     2007-05-19 11:29:32
 * Last modify: 2007-10-14 18:19:01 ivanovp {Time-stamp}
 * Copyright:   (C) Peter Ivanov, 2007
 * Licence:     GPL
 */
/**
 * \file lcd.h
 * \brief LCD handling functions header
 * \author Peter Ivanov
 */
#ifndef __INCLUDE_LCD_H
#define __INCLUDE_LCD_H

#define __FIX_PCB_BARGRAPH        // mettre  vrai si le programme doit tre gnr pour un 
                                  // PCB OLIMEX modifi. Modification  faire : 
                                  // couper les pattes 42 et 35 du LCD
                                  // retirer les rsistances R9 et R10
                                  // Connecter sur le PCB, 42 avec 40 et 35 avec 37.

#include <msp430x44x.h>
#include <inttypes.h>
#include "common.h"

#define LCD_PRINTF_BUF_SIZE     (8)
#define LCD_ARROW_UP            (0x20)
#define LCD_ARROW_DOWN          (0x80)
#define LCD_ARROW_RIGHT         (0x40)
#define LCD_ARROW_LEFT          (0x10)
#define LCD_ARROW_ALL           (LCD_ARROW_UP + LCD_ARROW_DOWN + LCD_ARROW_LEFT + LCD_ARROW_RIGHT)

/// Pointer to access LCD memory
extern char *LCD;

/**
 * Initializes LCD driver.
 *
 * @author Peter Ivanov, Muneem Shahriar
 */
void LCD_init ();

/**
 * Clear all segment of LCD.
 *
 * @author Peter Ivanov, Muneem Shahriar
 */
void LCD_clear ();

/**
 * Clear segments of clock on LCD (four digit at top-right).
 *
 * @author Peter Ivanov
 */
void LCD_clearClock ();

/**
 * Shows all segment of LCD. Useful for testing the LCD.
 *
 * @author Peter Ivanov, Muneem Shahriar
 */
void LCD_all ();

/**
 * Write an integer variable to clock part of LCD (four digit at top-right).
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param value Integer value to display: 0..9999.
 *   @param paddingZeroes Display zeroes if number of digits are less than 
 *                        four.
 */
void LCD_writeClockInt (uint16_t value, bool_t paddingZeroes);

/**
 * Write a digit to clock part of LCD (four digit at top-right).
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param position 0..3: 0 is the most right digit.
 *   @param digit 0..9: digit to display. -1: clear digit.
 */
void LCD_writeClockDigit (uint8_t position, uint8_t digit);

/**
 * Shows or hides colon on clock part of LCD (four digit at top-right).
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param on TRUE: switch on colon, FALSE: switch off colon.
 */
void LCD_setClockColon (bool_t on);

/**
 * Shows or hides dots on clock part of LCD (four digit at top-right).
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param position 0..2: 0 is the dot at most right digit.
 *   @param on TRUE: switch on dot, FALSE: switch off dot.
 */
void LCD_setClockDot (uint8_t position, bool_t on);



/**
 * Shows battery level. 
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param level Valid values are 0..3:
 *                0: Zero battery,
 *                1: Low battery,
 *                2: Medium battery,
 *                3: Full battery.
 */
void LCD_showBatteryLevel (uint8_t level);

/**
 * Shows or hides colon.
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param position 0..5: 0 is the dot at most right digit.
 *   @param on TRUE: switch on colon, FALSE: switch off colon.
 */
void LCD_setColon (uint8_t position, bool_t on);

/**
 * Write an integer variable to LCD.
 *
 * @author Peter Ivanov, Muneem Shahriar
 *
 * Input:
 *   @param number Integer value to display: 0..9999999.
 */
void LCD_writeInt (int number);

/**
 * Write an alphanumeric character to LCD.
 *
 * @author Peter Ivanov, Muneem Shahriar
 *
 * Input:
 *   @param position 0..5: 0 the most right digit.
 *   @param letter Implemented characters: A-Z 0-9 . : , / \
 */
void LCD_writeLetter (uint8_t position, char letter);

/**
 * Write a character buffer to LCD.
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param buf Pointer to buffer.
 *   @param length Length of buffer.
 */
void LCD_write (const char *buf, uint8_t length);
void LCD_writesp (const char *buf, uint8_t length);

/**
 * Write a formatted string to LCD.
 * Example:
<pre>
LCD_printf ("I %02i", i);
</pre>
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param fmt Printf format string. e.g.: "I %02i"
 */
void LCD_printf (const char *fmt, ...);
void LCD_printfsp (const char *fmt, ...);


/* affichage ou extinction du "-" */
void LCD_minus(bool_t on);


/* affichage ou extinction du "+" */
void LCD_plus(bool_t on);


/* Gestion du clignotement de l'afficheur */
void LCD_blink(bool_t on);


#ifdef __FIX_PCB_BARGRAPH
/*
Affiche les segments de la batterie 
level : 0  9 en fonction du niveau 
extremits  vrai affiche les [..]
Procdure utilisable uniquement si le PCB
a t modifi : suppression des rsistances
R10 et R9, coupure des pattes 42 et 35 de
l'afficheur LCD, strap entre les points 
(42 et 40) et (35 et 37) du PCB.
*/
void LCD_bargraph (uint8_t level,bool_t extremites);

#else // PCB pas modifi. On peut afficher les flches
/**
 * Shows arrows.
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param arrows Which arrow has to display: LCD_ARROW_UP | 
 *                 LCD_ARROW_DOWN | LCD_ARROW_RIGHT | 
 *                 LCD_ARROW_LEFT | LCD_ARROW_ALL.
 */
void LCD_showArrows (uint8_t arrows);

/**
 * Clear arrows.
 *
 * @author Peter Ivanov
 *
 * Input:
 *   @param arrows Which arrow has to clear: LCD_ARROW_UP | 
 *                 LCD_ARROW_DOWN | LCD_ARROW_RIGHT | 
 *                 LCD_ARROW_LEFT | LCD_ARROW_ALL.
 */
void LCD_clearArrows (uint8_t arrows);

#endif // __FIX_PCB_BARGRAPH

#endif // __INCLUDE_LCD_H
